/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.engine.omegat;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.omegat.core.data.StringEntry;
import org.omegat.gui.glossary.GlossaryEntry;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.core.BentenGlossarySearchResult;
import blanco.omegat.util.glossary.MyGlossaryManager;

/**
 * Benten システムのデフォルトの用語集エンジンです。
 * 
 * ★基本設計「共通機能: 用語対訳集エンジン機能」に対応します。
 */
public class DefaultGlossaryEngine implements BentenGlossaryEngine {
	/**
	 * OmegaT 用語集管理。
	 */
	private MyGlossaryManager fGm;

	/**
	 * コンストラクター。
	 */
	public DefaultGlossaryEngine() {
		fGm = new MyGlossaryManager();
	}

	/**
	 * {@inheritDoc}
	 */
	public void loadTsv(final File dirTsv) throws IOException {
		fGm.loadGlossaryFiles(dirTsv);
	}

	/**
	 * {@inheritDoc}
	 */
	public void unload() {
		fGm.clear();
	}

	/**
	 * {@inheritDoc}
	 */
	public List<BentenGlossarySearchResult> search(final String search) {
		final StringEntry strEntry = new StringEntry(search);
		final List<GlossaryEntry> glossaryEntryList = fGm.getGlossary(strEntry);

		final List<BentenGlossarySearchResult> result = new ArrayList<BentenGlossarySearchResult>();
		if (glossaryEntryList == null) {
			return result;
		}

		// 検索結果を API 形式に詰め替え。
		for (final GlossaryEntry glossaryEntry : glossaryEntryList) {
			final BentenGlossarySearchResult row = new BentenGlossarySearchResult();
			row.setSource(glossaryEntry.getSrcText());
			row.setTarget(glossaryEntry.getLocText());
			row.setNote(glossaryEntry.getCommentText());
			result.add(row);
		}
		return result;
	}
}
