package benten.cat.glossary.engine.omegat.ui;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryDriver;
import benten.cat.glossary.core.BentenGlossaryDriverManager;
import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.data.AbstractBentenGlossaryDataTsvPlugin;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.cat.glossary.engine.omegat.DefaultGlossaryDriver;

/**
 * Benten システムのデフォルトの用語集エンジン・プラグインである OmegaT 用語集プラグインです。
 * 
 * <UL>
 * <LI>benten.cat.glossary.engine.omegat を Eclipse にプラグします。
 * </UL>
 * 
 * ★基本設計「共通機能: 用語対訳集エンジン機能」に対応します。
 */
public class OmegatGlossaryPlugin extends AbstractBentenGlossaryEnginePlugin {
	private static final String DEFAULT_GLOSSARY = "Default Glossary";

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenGlossaryDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.glossary.engine.omegat.DefaultGlossaryDriver");
		} catch (final ClassNotFoundException e) {
			log("デフォルトのグロッサリー・エンジンの読み込みに失敗しました。");
			log(e);
		}

		return BentenGlossaryDriverManager.getDriver(DefaultGlossaryDriver.DRIVER_NAME);
	}

	@Override
	public BentenGlossaryEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}
		return fEngine;
	}

	@SuppressWarnings("static-access")
	@Override
	public void load() {
		getDefault().log("OmegaT 用語集エンジン・プラグイン: 用語集をリフレッシュ・ロードします。");

		getEngine().unload();

		final Map<String, AbstractBentenGlossaryDataTsvPlugin> glossaryMap = new HashMap<String, AbstractBentenGlossaryDataTsvPlugin>();

		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.data.tsv");
		if (point == null) {
			return;
		}

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name");
					final Object obj = cfgElem.createExecutableExtension("class");
					if (obj instanceof AbstractBentenGlossaryDataTsvPlugin) {
						glossaryMap.put(name, (AbstractBentenGlossaryDataTsvPlugin) obj);
					}
				} catch (final CoreException e) {
					OmegatGlossaryPlugin.getDefault().log(e);
				}
			}
		}

		final AbstractBentenGlossaryDataTsvPlugin glossaryDataPlugin = glossaryMap.get(DEFAULT_GLOSSARY);
		if (glossaryDataPlugin == null) {
			return;
		}

		try {
			// FIXME ファイルの読み込み方
			final URL entry = glossaryDataPlugin.getDefault().getBundle().getEntry("/glossary");
			final String pluginDirectory = FileLocator.resolve(entry).getPath();
			final File file = new File(pluginDirectory);
			getEngine().loadTsv(file);
		} catch (final IOException e) {
			getDefault().log(e);
		}
	}
}
